/**
 *
 * Project: Eight Track, File: TrackScheduler.java
 *
 * Copyright 2019-2019 Locusworks LLC.
 * All rights reserved. Federal copyright law prohibits unauthorized reproduction by
 * any means and imposes fines up to $25,000 for violation.  No part of this material
 * may be reproduced, transmitted, transcribed, stored in a retrieval system, copied,
 * modified, duplicated, adapted or translated into another program language in any
 * form or by any means, electronic, mechanical, photocopying, recording, or
 * otherwise, without the prior written permission from Locusworks. Locusworks
 * affirms that Eight-Track(R) software and data is subject to United States
 * Government Purpose Rights.  Contact Locusworks, 1313 Lawnview Drive
 * Forney TX 75126, (802) 488-0438, for commercial licensing opportunities.
 *
 * IN NO EVENT SHALL LOCUSWORKS BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LOCUSWORKS HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  NO RESPONSIBILITY IS ASSUMED BY
 * LOCUSWORKS FOR ITS USE, OR FOR ANY INFRINGEMENTS OF PATENTS OR OTHER RIGHTS OF
 * THIRD PARTIES RESULTING FROM ITS USE.  LOCUSWORKS SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND
 * ACCOMPANYING DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS
 * IS". LOCUSWORKS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 */
package net.locusworks.discord.eighttrack.audio;

import java.util.Deque;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.function.Consumer;

import com.sedmelluq.discord.lavaplayer.player.AudioLoadResultHandler;
import com.sedmelluq.discord.lavaplayer.player.AudioPlayer;
import com.sedmelluq.discord.lavaplayer.player.event.AudioEventAdapter;
import com.sedmelluq.discord.lavaplayer.player.event.AudioEventListener;
import com.sedmelluq.discord.lavaplayer.tools.FriendlyException;
import com.sedmelluq.discord.lavaplayer.track.AudioPlaylist;
import com.sedmelluq.discord.lavaplayer.track.AudioTrack;
import com.sedmelluq.discord.lavaplayer.track.AudioTrackEndReason;

public class TrackScheduler extends AudioEventAdapter implements AudioEventListener, AudioLoadResultHandler  {
  
  private Deque<AudioTrack> trackQueue;
  
  private boolean started;
  
  private Consumer<AudioTrackEndReason> finished;
  
  public TrackScheduler() {
    trackQueue = new LinkedBlockingDeque<AudioTrack>();
  }
  
  @Override
  public void onTrackStart(AudioPlayer player, AudioTrack track) {
    this.started = true;
    super.onTrackStart(player, track);
  }
  
  @Override
  public void onTrackEnd(AudioPlayer player, AudioTrack track, AudioTrackEndReason endReason) {
    this.started = false;
    super.onTrackEnd(player, track, endReason);
    if (finished != null) finished.accept(endReason);
  }
  
  public void setFinishedCallback(Consumer<AudioTrackEndReason> callback) {
    if (this.finished == null) this.finished = callback;
  }

  public void trackLoaded(AudioTrack track) {
    trackQueue.addFirst(track);
  }

  public void playlistLoaded(AudioPlaylist playlist) {
    for (AudioTrack at : playlist.getTracks()) {
      trackQueue.add(at);
    }
  }
  
  public void clearTracks() {
    trackQueue.clear();
  }

  public void noMatches() {
    // TODO Auto-generated method stub
  }

  public void loadFailed(FriendlyException exception) {
    // TODO Auto-generated method stub
  }
  
  public int trackCount() {
    return trackQueue.size();
  }
  
  public AudioTrack peek() {
    return trackQueue.peek();
  }
  
  public boolean hasTracks() {
    return !trackQueue.isEmpty();
  }
  
  public AudioTrack getNextTrack() {
    return trackQueue.poll();
  }
  
  public boolean playing() {
    return started;
  }

}
