/**
 *
 * Project: Eight Track, File: SongListHandler.java
 *
 * Copyright 2019-2020 Locusworks LLC.
 * All rights reserved. Federal copyright law prohibits unauthorized reproduction by
 * any means and imposes fines up to $25,000 for violation.  No part of this material
 * may be reproduced, transmitted, transcribed, stored in a retrieval system, copied,
 * modified, duplicated, adapted or translated into another program language in any
 * form or by any means, electronic, mechanical, photocopying, recording, or
 * otherwise, without the prior written permission from Locusworks. Locusworks
 * affirms that Eight-Track(R) software and data is subject to United States
 * Government Purpose Rights.  Contact Locusworks, 1313 Lawnview Drive
 * Forney TX 75126, (802) 488-0438, for commercial licensing opportunities.
 *
 * IN NO EVENT SHALL LOCUSWORKS BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LOCUSWORKS HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  NO RESPONSIBILITY IS ASSUMED BY
 * LOCUSWORKS FOR ITS USE, OR FOR ANY INFRINGEMENTS OF PATENTS OR OTHER RIGHTS OF
 * THIRD PARTIES RESULTING FROM ITS USE.  LOCUSWORKS SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND
 * ACCOMPANYING DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS
 * IS". LOCUSWORKS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 */
package net.locusworks.discord.eighttrack.events.main.handlers;

import java.util.List;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.locusworks.discord.eighttrack.database.entities.GuildSong;
import net.locusworks.discord.eighttrack.events.main.AbstractMainEventHandler;
import net.locusworks.discord.eighttrack.handlers.GuildMusicHandler;
import net.locusworks.discord.eighttrack.services.RepositoryService;
import net.locusworks.logger.ApplicationLogger;
import net.locusworks.logger.ApplicationLoggerFactory;

@Component
public class SongListHandler extends AbstractMainEventHandler {

  private ApplicationLogger logger;
  
  @Autowired
  private RepositoryService guildRepoService;
  
  public SongListHandler() {
    super("-list");
    logger = ApplicationLoggerFactory.getLogger(SongListHandler.class);
  }

  @Override
  public void executeEvent(GuildMusicHandler musicHandler, GuildMessageReceivedEvent event, List<String> commands) {
    try {
      listSongs(event, commands);
    } catch (Exception ex) {
      logger.error("Unable to play next song: " + ex.getMessage(), ex);
      guildRepoService.logEntry(event.getGuild().getIdLong(), ex, "Unable to play next song: %s", ex.getMessage());
      sendMessage(event, "Sorry I am unable to play next song: " + ex.getMessage());
    }
  }
  
  private void listSongs(GuildMessageReceivedEvent event, List<String> commands) throws Exception {
    List<GuildSong> gsList = guildRepoService.getGuildSongRepo().findByGuild(event.getGuild().getIdLong());

    if (gsList == null || gsList.isEmpty()) {
      sendMessage(event, "There is no music for this guild.");
      return;
    }

    int longestSong = 0;
    int longestArtist = 0;
    for(GuildSong gs : gsList) {
      if (gs.getSong().getTitle().length() > longestSong) longestSong = gs.getSong().getTitle().length();
      if (gs.getSong().getArtist().length() > longestArtist) longestArtist = gs.getSong().getArtist().length();
    }
    
    sendMessage(event, true, "**Currently available songs for %s**\n\n", event.getGuild().getName());

    int count = 0;
    String fmt = "%6s | %-" + longestSong +"s | %-" + longestArtist +"s | %s%n";
    StringBuilder sb = new StringBuilder("```");
    sb.append(String.format(fmt, "Track", "Title", "Artist", "id"));
    sb.append(StringUtils.repeat("-", 27 + longestSong + longestArtist)).append("\n");

    for(GuildSong gs : gsList) {
      String msg = String.format(fmt, ++count, gs.getSong().getTitle(), gs.getSong().getArtist(), gs.getUuid());
      if (sb.length() + msg.length() > 2000) {
        sb.append("```");
        sendMessage(event, sb.toString(), true);
        sb = new StringBuilder("```");
      } else {
        sb.append(msg);
      }
    }
    sb.append("```");
    sendMessage(event, sb.toString());
  }

  @Override
  public String help() {
    return "-list (lists the sonds to play)";
  }

}
