/**
 *
 * Project: Eight Track, File: PlaylistAddHandler.java
 *
 * Copyright 2019-2020 Locusworks LLC.
 * All rights reserved. Federal copyright law prohibits unauthorized reproduction by
 * any means and imposes fines up to $25,000 for violation.  No part of this material
 * may be reproduced, transmitted, transcribed, stored in a retrieval system, copied,
 * modified, duplicated, adapted or translated into another program language in any
 * form or by any means, electronic, mechanical, photocopying, recording, or
 * otherwise, without the prior written permission from Locusworks. Locusworks
 * affirms that Eight-Track(R) software and data is subject to United States
 * Government Purpose Rights.  Contact Locusworks, 1313 Lawnview Drive
 * Forney TX 75126, (802) 488-0438, for commercial licensing opportunities.
 *
 * IN NO EVENT SHALL LOCUSWORKS BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LOCUSWORKS HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  NO RESPONSIBILITY IS ASSUMED BY
 * LOCUSWORKS FOR ITS USE, OR FOR ANY INFRINGEMENTS OF PATENTS OR OTHER RIGHTS OF
 * THIRD PARTIES RESULTING FROM ITS USE.  LOCUSWORKS SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND
 * ACCOMPANYING DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS
 * IS". LOCUSWORKS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 */
package net.locusworks.discord.eighttrack.events.playlist.handlers;

import java.awt.Color;
import java.time.OffsetDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.MessageEmbed;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.locusworks.discord.eighttrack.database.entities.DiscordGuild;
import net.locusworks.discord.eighttrack.database.entities.GuildPlaylist;
import net.locusworks.discord.eighttrack.database.entities.GuildPlaylistSong;
import net.locusworks.discord.eighttrack.database.entities.GuildSong;
import net.locusworks.discord.eighttrack.events.playlist.AbstractPlaylistEventHandler;
import net.locusworks.discord.eighttrack.handlers.GuildMusicHandler;
import net.locusworks.discord.eighttrack.services.RepositoryService;

@Component
public class PlaylistAddHandler extends AbstractPlaylistEventHandler {
  
  @Autowired
  private RepositoryService guildRepoService;

  public PlaylistAddHandler() {
    super("add");
  }

  @Override
  public void executeEvent(GuildMusicHandler musicHandler, GuildMessageReceivedEvent event, List<String> commands) {
    if(!checkCommands(event, commands, 
        event.getAuthor().getAsMention() + " you have to provide the playlist to create a new playlist (no spaces in name) and optionally a list of uuids for songs to add")) return;

    Long guildId = event.getGuild().getIdLong();

    DiscordGuild guild = guildRepoService.getGuildRepo().findByGuildId(guildId);
    if (guild == null) {
      event.getChannel().sendMessage("Unable to find guild in local database. Please contact administrator").queue();
      return;
    }

    String playlist = commands.remove(0);

    long userId = event.getMember().getIdLong();

    GuildPlaylist gpl = guildRepoService.getGuildPlaylistRepo().findByGuildAndUserIdAndPlaylist(guild, userId, playlist);
    boolean newList = false;
    if (gpl != null && commands.isEmpty()) {
      event.getChannel().sendMessage(event.getAuthor().getAsMention() + " a playlist with the name " + playlist + " already exist for you.").queue();
      return;
    } else if (gpl == null) {
      newList = true;
      gpl = new GuildPlaylist();
      gpl.setDateAdded(new Date());
      gpl.setGuild(guild);
      gpl.setPlaylist(playlist);
      gpl.setUserId(userId);
      guildRepoService.getGuildPlaylistRepo().save(gpl);
    }

    if (commands.isEmpty() && newList) {
      event.getChannel().sendMessage(event.getAuthor().getAsMention() + " playlist " + playlist + " successfully created.").queue();
      return;
    }

    Set<String> songIds = guildRepoService.getGuildPlaylistSongRepo()
        .findByGuildPlaylistAndSongIds(gpl, commands).stream()
        .map(g -> g.getGuildSong().getUuid())
        .collect(Collectors.toSet());

    List<GuildPlaylistSong> gplsList = new ArrayList<>();
    StringBuilder sb = new StringBuilder();
    for(String id : commands.stream().collect(Collectors.toSet())) {
      GuildSong gs = guildRepoService.getGuildSongRepo().findByUuid(id);
      if (songIds.contains(id)) {
        sb.append(String.format("**%s** by __%s__ already exists in this playlist.%n", gs.getSong().getTitle(), gs.getSong().getArtist()));
        continue;
      }

      if (gs == null) {
        sb.append("Song with id `" + id + "` not found. Please check id\n");
        continue;
      }

      GuildPlaylistSong gpls = new GuildPlaylistSong();
      gpls.setDateAdded(new Date());
      gpls.setGuildPlaylist(gpl);
      gpls.setGuildSong(gs);
      gplsList.add(gpls);

      sb.append(String.format("**%s** by __%s__ added to playlist.%n", gs.getSong().getTitle(), gs.getSong().getArtist()));
    }

    if (!gplsList.isEmpty()) {
      guildRepoService.getGuildPlaylistSongRepo().saveAll(gplsList);
    }

    MessageEmbed embed = new EmbedBuilder()
        .setTitle("Results for adding playlist " + playlist)
        .setAuthor(event.getMember().getEffectiveName(), null, event.getAuthor().getAvatarUrl())
        .setColor(Color.GREEN)
        .setDescription(sb.toString())
        .setFooter("", event.getGuild().getSelfMember().getUser().getAvatarUrl())
        .setTimestamp(OffsetDateTime.now())
        .build();

    event.getChannel().sendMessage(embed).queue();
  }

  @Override
  public String help() {
    // TODO Auto-generated method stub
    return null;
  }

}
