/**
 *
 * Project: Eight Track, File: PlaylistDeleteHandler.java
 *
 * Copyright 2019-2020 Locusworks LLC.
 * All rights reserved. Federal copyright law prohibits unauthorized reproduction by
 * any means and imposes fines up to $25,000 for violation.  No part of this material
 * may be reproduced, transmitted, transcribed, stored in a retrieval system, copied,
 * modified, duplicated, adapted or translated into another program language in any
 * form or by any means, electronic, mechanical, photocopying, recording, or
 * otherwise, without the prior written permission from Locusworks. Locusworks
 * affirms that Eight-Track(R) software and data is subject to United States
 * Government Purpose Rights.  Contact Locusworks, 1313 Lawnview Drive
 * Forney TX 75126, (802) 488-0438, for commercial licensing opportunities.
 *
 * IN NO EVENT SHALL LOCUSWORKS BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LOCUSWORKS HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  NO RESPONSIBILITY IS ASSUMED BY
 * LOCUSWORKS FOR ITS USE, OR FOR ANY INFRINGEMENTS OF PATENTS OR OTHER RIGHTS OF
 * THIRD PARTIES RESULTING FROM ITS USE.  LOCUSWORKS SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND
 * ACCOMPANYING DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS
 * IS". LOCUSWORKS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 */
package net.locusworks.discord.eighttrack.events.playlist.handlers;

import java.awt.Color;
import java.time.OffsetDateTime;
import java.util.List;
import java.util.concurrent.TimeUnit;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.MessageEmbed;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.locusworks.discord.eighttrack.database.entities.DiscordGuild;
import net.locusworks.discord.eighttrack.database.entities.GuildPlaylist;
import net.locusworks.discord.eighttrack.database.entities.GuildPlaylistSong;
import net.locusworks.discord.eighttrack.database.entities.GuildSong;
import net.locusworks.discord.eighttrack.events.playlist.AbstractPlaylistEventHandler;
import net.locusworks.discord.eighttrack.handlers.GuildMusicHandler;
import net.locusworks.discord.eighttrack.handlers.ReactionHandler;
import net.locusworks.discord.eighttrack.listeners.ReactionListener;
import net.locusworks.discord.eighttrack.services.RepositoryService;
import net.locusworks.discord.eighttrack.utils.Reactions;
import net.locusworks.logger.ApplicationLogger;
import net.locusworks.logger.ApplicationLoggerFactory;

@Component
public class PlaylistDeleteHandler extends AbstractPlaylistEventHandler {
  
  @Autowired
  private RepositoryService guildRepoService;
  
  @Autowired
  private ReactionHandler reactionHandler;
  
  private ApplicationLogger logger;
  public PlaylistDeleteHandler() {
    super("delete");
    logger = ApplicationLoggerFactory.getLogger(PlaylistDeleteHandler.class);
  }

  @Override
  public void executeEvent(GuildMusicHandler musicHandler, GuildMessageReceivedEvent event, List<String> commands) {
      if(!checkCommands(event, commands, 
          event.getAuthor().getAsMention() + " you have to provide the playlist name and optionally a list of uuids for songs to remove")) return;

      Long guildId = event.getGuild().getIdLong();

      DiscordGuild guild = guildRepoService.getGuildRepo().findByGuildId(guildId);
      if (guild == null) {
        event.getChannel().sendMessage("Unable to find guild in local database. Please contact administrator").queue();
        return;
      }

      String playlist = commands.remove(0);

      long userId = event.getMember().getIdLong();

      GuildPlaylist gpl = guildRepoService.getGuildPlaylistRepo().findByGuildAndUserIdAndPlaylist(guild, userId, playlist);

      if (commands.isEmpty()) {

        MessageEmbed embed = new EmbedBuilder()
            .setColor(Color.RED)
            .setTitle("Delete Playlist " + playlist)
            .setDescription("Are you sure you want to delete the playlist " + playlist)
            .setTimestamp(OffsetDateTime.now())
            .setFooter(event.getGuild().getSelfMember().getEffectiveName(), event.getGuild().getSelfMember().getUser().getAvatarUrl())
            .build();

        event.getChannel().sendMessage(embed).queue((msg) -> {

          ReactionListener<String> handler = new ReactionListener<>(userId, msg.getId());
          handler.setExpiresIn(TimeUnit.MINUTES, 1);
          handler.registerReaction(Reactions.CHECK_MARK_BUTTON, (ret) -> deletePlayListConfirm(gpl, event.getMember().getAsMention(), msg));
          handler.registerReaction(Reactions.CROSS_MARK_BUTTON, (ret) -> deleteMessage(msg, reactionHandler, event.getGuild().getIdLong()));

          reactionHandler.addReactionListener(guildId, msg, handler);
        });
        return;
      }

      List<GuildPlaylistSong> songs = guildRepoService.getGuildPlaylistSongRepo().findByGuildPlaylistAndSongIds(gpl, commands);

      StringBuilder sb = new StringBuilder("Are you sure you want to delete the following songs from playlist " + playlist + "?\n");
      for (GuildPlaylistSong gpls : songs) {
        GuildSong gs = gpls.getGuildSong();
        sb.append(String.format("**%s** by __%s__%n", gs.getSong().getTitle(), gs.getSong().getArtist()));
      }

      MessageEmbed embed = new EmbedBuilder()
          .setColor(Color.RED)
          .setTitle("Delete Playlist Songs")
          .setDescription(sb.toString())
          .setTimestamp(OffsetDateTime.now())
          .setFooter(event.getGuild().getSelfMember().getEffectiveName(), event.getGuild().getSelfMember().getUser().getAvatarUrl())
          .build();

      event.getChannel().sendMessage(embed).queue((msg) -> {

        ReactionListener<String> handler = new ReactionListener<>(userId, msg.getId());
        handler.setExpiresIn(TimeUnit.MINUTES, 1);
        handler.registerReaction(Reactions.CHECK_MARK_BUTTON, (ret) -> deletePlayListSongConfirm(songs, event.getMember().getAsMention(), msg));
        handler.registerReaction(Reactions.CROSS_MARK_BUTTON, (ret) -> deleteMessage(msg, reactionHandler, event.getGuild().getIdLong()));

        reactionHandler.addReactionListener(guildId, msg, handler);
      });
  }
  
  private void deletePlayListSongConfirm(List<GuildPlaylistSong> songs, String user, Message msg) {
    try {
      guildRepoService.getGuildPlaylistSongRepo().deleteAll(songs);
      msg.getChannel().sendMessage(user + ", songs removed successfully fom playlist").complete();
    } catch (Exception ex) {
      msg.getChannel().sendMessage("Sorry " + user + " I was unable to remove songs from the playlist. Reason: " + ex.getMessage()).complete();
      logger.error("Unable to delete songs from playlist : " + ex.getMessage());
      logger.error(ex);
      guildRepoService.logEntry(msg.getGuild().getIdLong(), ex, "Unable to delete playlist: %s", ex.getMessage());
    } finally {
      deleteMessage(msg, reactionHandler, msg.getGuild().getIdLong());
    }
  }

  private void deletePlayListConfirm(GuildPlaylist gpl, String user, Message msg) {
    String playlist = gpl.getPlaylist();
    try {
      guildRepoService.getGuildPlaylistRepo().delete(gpl);
      msg.getChannel().sendMessage(user + " " + playlist  + " deleted successfully").complete();
    } catch (Exception ex) {
      msg.getChannel().sendMessage("Sorry " + user + " I was unable to remove playlist " + playlist + ". Reason: " + ex.getMessage()).complete();
      logger.error("Unable to delete playlist " + playlist + ": " + ex.getMessage());
      logger.error(ex);
      guildRepoService.logEntry(msg.getGuild().getIdLong(), ex, "Unable to delete playlist: %s", ex.getMessage());
    } finally {
      deleteMessage(msg, reactionHandler, msg.getGuild().getIdLong());
    }
  }

  @Override
  public String help() {
    // TODO Auto-generated method stub
    return null;
  }

}
